@@ -23,8 +23,6 @@ from lib389.replica import Replicas, Changelog
 from lib389.cos import (CosTemplates, CosIndirectDefinitions,
                         CosPointerDefinitions, CosClassicDefinitions)
 
-# We need to be a factor to the backend monitor
-from lib389.monitor import MonitorBackend
 from lib389.index import Index, Indexes, VLVSearches, VLVSearch
 from lib389.tasks import ImportTask, ExportTask, Tasks
 from lib389.encrypted_attributes import EncryptedAttr, EncryptedAttrs
@@ -527,7 +525,7 @@ class Backend(DSLdapObject):
                 yield report
         except:
             # Suffix is not replicated
-            self.log.debug(f"_lint_cl_trimming - backend ({suffix}) is not replicated")
+            self._log.debug(f"_lint_cl_trimming - backend ({suffix}) is not replicated")
             pass
 
     def create_sample_entries(self, version):
@@ -694,6 +692,8 @@ class Backend(DSLdapObject):
 
     def get_monitor(self):
         """Get a MonitorBackend(DSLdapObject) for the backend"""
+        # We need to be a factor to the backend monitor
+        from lib389.monitor import MonitorBackend
 
         monitor = MonitorBackend(instance=self._instance, dn="cn=monitor,%s" % self._dn)
         return monitor
@@ -719,7 +719,7 @@ class Backend(DSLdapObject):
                 return
         raise ValueError("Can not delete index because it does not exist")
 
-    def add_index(self, attr_name, types, matching_rules=[], reindex=False):
+    def add_index(self, attr_name, types, matching_rules=None, reindex=False):
         """ Add an index.
 
         :param attr_name - name of the attribute to index
@@ -736,7 +736,9 @@ class Backend(DSLdapObject):
             mrs = []
             for mr in matching_rules:
                 mrs.append(mr)
-            props['nsMatchingRule'] = mrs
+            # Only add if there are actually rules present in the list.
+            if len(mrs) > 0:
+                props['nsMatchingRule'] = mrs
         new_index.create(properties=props, basedn="cn=index," + self._dn)
 
         if reindex:
@@ -865,6 +867,37 @@ class Backend(DSLdapObject):
     def get_cos_templates(self):
         return CosTemplates(self._instance, self._dn).list()
 
+    def get_state(self):
+        suffix = self.get_attr_val_utf8('nsslapd-suffix')
+        try:
+            mt = self._mts.get(suffix)
+        except ldap.NO_SUCH_OBJECT:
+            raise ValueError("Backend missing mapping tree entry, unable to get state")
+        return mt.get_attr_val_utf8('nsslapd-state')
+
+    def set_state(self, new_state):
+        new_state = new_state.lower()
+        suffix = self.get_attr_val_utf8('nsslapd-suffix')
+        try:
+            mt = self._mts.get(suffix)
+        except ldap.NO_SUCH_OBJECT:
+            raise ValueError("Backend missing mapping tree entry, unable to set configuration")
+
+        if new_state not in ['backend', 'disabled',  'referral',  'referral on update']:
+            raise ValueError(f"Invalid backend state {new_state}, value must be one of the following: 'backend', 'disabled',  'referral',  'referral on update'")
+
+        # Can not change state of replicated backend
+        replicas = Replicas(self._instance)
+        try:
+            # Check if replication is enabled
+            replicas.get(suffix)
+            raise ValueError("Can not change the backend state of a replicated suffix")
+        except ldap.NO_SUCH_OBJECT:
+            pass
+
+        # Ok, change the state
+        mt.replace('nsslapd-state', new_state)
+
 
 class Backends(DSLdapObjects):
     """DSLdapObjects that represents DN_LDBM base DN
@@ -1015,6 +1048,9 @@ class DatabaseConfig(DSLdapObject):
             'nsslapd-rangelookthroughlimit',
             'nsslapd-backend-opt-level',
             'nsslapd-backend-implement',
+            'nsslapd-db-durable-transaction',
+            'nsslapd-search-bypass-filter-test',
+            'nsslapd-serial-lock',
         ]
         self._db_attrs = {
             'bdb':
@@ -1022,37 +1058,42 @@ class DatabaseConfig(DSLdapObject):
                     'nsslapd-dbcachesize',
                     'nsslapd-db-logdirectory',
                     'nsslapd-db-home-directory',
-                    'nsslapd-db-durable-transaction',
                     'nsslapd-db-transaction-wait',
                     'nsslapd-db-checkpoint-interval',
                     'nsslapd-db-compactdb-interval',
+                    'nsslapd-db-compactdb-time',
                     'nsslapd-db-page-size',
                     'nsslapd-db-transaction-batch-val',
                     'nsslapd-db-transaction-batch-min-wait',
                     'nsslapd-db-transaction-batch-max-wait',
                     'nsslapd-db-logbuf-size',
                     'nsslapd-db-locks',
+                    'nsslapd-db-locks-monitoring-enabled',
+                    'nsslapd-db-locks-monitoring-threshold',
+                    'nsslapd-db-locks-monitoring-pause',
                     'nsslapd-db-private-import-mem',
                     'nsslapd-import-cache-autosize',
                     'nsslapd-cache-autosize',
                     'nsslapd-cache-autosize-split',
                     'nsslapd-import-cachesize',
-                    'nsslapd-search-bypass-filter-test',
-                    'nsslapd-serial-lock',
                     'nsslapd-db-deadlock-policy',
                 ],
-            'lmdb': []
+            'mdb': [
+                    'nsslapd-mdb-max-size',
+                    'nsslapd-mdb-max-readers',
+                    'nsslapd-mdb-max-dbs',
+                ]
         }
         self._create_objectclasses = ['top', 'extensibleObject']
         self._protected = True
-        # This could be "bdb" or "lmdb", use what we have configured in the global config
+        # This could be "bdb" or "mdb", use what we have configured in the global config
         self._db_lib = self.get_attr_val_utf8_l('nsslapd-backend-implement')
         self._dn = "cn=config,cn=ldbm database,cn=plugins,cn=config"
         self._db_dn = f"cn={self._db_lib},cn=config,cn=ldbm database,cn=plugins,cn=config"
         self._globalObj = DSLdapObject(self._instance, dn=self._dn)
         self._dbObj = DSLdapObject(self._instance, dn=self._db_dn)
         # Assert there is no overlap in different config sets
-        assert_c(len(set(self._global_attrs).intersection(set(self._db_attrs['bdb']), set(self._db_attrs['lmdb']))) == 0)
+        assert_c(len(set(self._global_attrs).intersection(set(self._db_attrs['bdb']), set(self._db_attrs['mdb']))) == 0)
 
     def get(self):
         """Get the combined config entries"""
@@ -1075,7 +1116,7 @@ class DatabaseConfig(DSLdapObject):
             self._instance.log.info(f'{k}: {vo}')
 
     def get_db_lib(self):
-        """Return the backend library, bdb, lmdb, etc"""
+        """Return the backend library, bdb, mdb, etc"""
         return self._db_lib
 
     def set(self, value_pairs):
@@ -1087,8 +1128,9 @@ class DatabaseConfig(DSLdapObject):
             elif attr in self._db_attrs['bdb']:
                 db_config = DSLdapObject(self._instance, dn=self._db_dn)
                 db_config.replace(attr, val)
-            elif attr in self._db_attrs['lmdb']:
-                pass
+            elif attr in self._db_attrs['mdb']:
+                db_config = DSLdapObject(self._instance, dn=self._db_dn)
+                db_config.replace(attr, val)
             else:
                 # Unknown attribute
                 raise ValueError("Can not update database configuration with unknown attribute: " + attr)
