@@ -1,5 +1,5 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2019 Red Hat, Inc.
+# Copyright (C) 2020 Red Hat, Inc.
 # Copyright (C) 2017, William Brown <william at blackhats.net.au>
 # All rights reserved.
 #
@@ -12,14 +12,14 @@ import time
 import subprocess
 from enum import Enum
 import ldap
-
 from lib389._mapped_object import DSLdapObject, DSLdapObjects, _gen_or, _gen_filter, _term_gen
 from lib389._constants import SER_ROOT_DN, SER_ROOT_PW
-from lib389.utils import ds_is_older, gentime_to_posix_time, gentime_to_datetime
+from lib389.utils import gentime_to_posix_time, gentime_to_datetime
 from lib389.plugins import AccountPolicyPlugin, AccountPolicyConfig, AccountPolicyEntry
 from lib389.cos import CosTemplates
 from lib389.mappingTree import MappingTrees
 from lib389.idm.role import Roles
+from lib389.extended_operations import LdapSSOTokenRequest, LdapSSOTokenResponse
 
 
 class AccountState(Enum):
@@ -44,9 +44,13 @@ class Account(DSLdapObject):
     :type dn: str
     """
 
+    def __init__(self, instance, dn=None):
+        super(Account, self).__init__(instance, dn)
+        self._protected = False
+
     def _format_status_message(self, message, create_time, modify_time, last_login_time, limit, role_dn=None):
         params = {}
-        now = time.time()
+        now = time.mktime(time.gmtime())
         params["Creation Date"] = gentime_to_datetime(create_time)
         params["Modification Date"] = gentime_to_datetime(modify_time)
         params["Last Login Date"] = None
@@ -84,6 +88,7 @@ class Account(DSLdapObject):
 
         # Fetch Account Policy data if its enabled
         plugin = AccountPolicyPlugin(inst)
+        config_dn = plugin.get_attr_val_utf8("nsslapd-pluginarg0")
         state_attr = ""
         alt_state_attr = ""
         limit = ""
@@ -95,8 +100,7 @@ class Account(DSLdapObject):
         except IndexError:
             self._log.debug("The bound user doesn't have rights to access Account Policy settings. Not checking.")
 
-        if process_account_policy:
-            config_dn = plugin.get_attr_val_utf8("nsslapd-pluginarg0")
+        if process_account_policy and config_dn is not None:
             config = AccountPolicyConfig(inst, config_dn)
             config_settings = config.get_attrs_vals_utf8(["stateattrname", "altstateattrname",
                                                           "specattrname", "limitattrname"])
@@ -105,7 +109,9 @@ class Account(DSLdapObject):
             spec_attr = self._dict_get_with_ignore_indexerror(config_settings, "specattrname")
             limit_attr = self._dict_get_with_ignore_indexerror(config_settings, "limitattrname")
 
-            cos_entries = CosTemplates(inst, self.dn)
+            mapping_trees = MappingTrees(inst)
+            root_suffix = mapping_trees.get_root_suffix_by_entry(self.dn)
+            cos_entries = CosTemplates(inst, root_suffix)
             accpol_entry_dn = ""
             for cos in cos_entries.list():
                 if cos.present(spec_attr):
@@ -157,8 +163,8 @@ class Account(DSLdapObject):
 
         # Locked indirectly through Account Policy plugin
         if process_account_policy and last_login_time:
-            # Now check the Acount Policy Plugin inactivity limits
-            remaining_time = float(limit) - (time.time() - gentime_to_posix_time(last_login_time))
+            # Now check the Account Policy Plugin inactivity limits
+            remaining_time = float(limit) - (time.mktime(time.gmtime()) - gentime_to_posix_time(last_login_time))
             if remaining_time <= 0:
                 return self._format_status_message(AccountState.INACTIVITY_LIMIT_EXCEEDED,
                                                    create_time, modify_time, last_login_time, limit)
@@ -292,6 +298,34 @@ class Account(DSLdapObject):
         self._instance.passwd_s(self._dn, current_password, new_password,
             serverctrls=self._server_controls, clientctrls=self._client_controls, escapehatch='i am sure')
 
+    def request_sso_token(self):
+        """From an authenticated connection, request a single sign on token (cookie) for
+        future use.
+
+        Note this function will fail if the connection was itself authenticated with a
+        token - this is to prevent token renewal from being infinite which may be a
+        security risk.
+
+        :returns: String of the token for use with authenticate_sso_token
+        """
+        # Build the request
+        req = LdapSSOTokenRequest()
+        # Get the response
+        (_, res) = self._instance.extop_s(req, serverctrls=self._server_controls,
+            clientctrls=self._client_controls, escapehatch='i am sure')
+        (lt, token) = LdapSSOTokenResponse(res).responseValue
+        return token
+
+    def authenticate_sso_token(self, token, *args, **kwargs):
+        """
+        Given an entry (with dn), bind using this authentication token.
+
+        :param token: An entry token
+        :type token: str
+        :returns: Connection with a binding as the entry
+        """
+        return self.bind(token, *args, **kwargs)
+
 
 class Accounts(DSLdapObjects):
     """DSLdapObjects that represents Account entry
